#!/usr/bin/ksh
#
# ###################################################################
# migrate_uid.ksh - Copyright (c) 2002 MonkeyMental.com
# ###################################################################
# Description:
# This script will migrate old UIDs to new UIDs based on a uid map.
#
# The uidmap format is as follows:
# olduid<tab>newuid
#
# This script expects a filename as an argument (probably from a find
# command.)
# 
# We will perform the following actions:
#  1.) Log the current state of the file
#  2.) Determine the current UID of the file
#  3.) Determine if that UID is a local or NIS account
#  4.) If it's local, leave file unchanged
#  5.) If it's a NIS account, lookup new UID in uidmap and chown file
#  6.) Log any changes to files for later inspection
# ###################################################################
# Revision History: 
# 02-01-16  LO	Created.
# 02-01-17  LO	Added 2nd command line argument to be mount we are
#				working on.
# 02-01-17	LO	Got logging working.
# 02-01-17	LO	Fixed case prob. Thanks Vic.
# 02-05-21	LO	Added Linux as a supported OS.
# ###################################################################

###################################
# Setup some generic variables
###################################

# File we are working on (a.k.a. the Target)
TARGET=${1}
# Mount point we are working on.
MOUNT=${2}

# Log location
LOG_DIR="/usr/local/migrate";
ORIG_LOG="${LOG_DIR}/${MOUNT}-save.log";
CHNG_LOG="${LOG_DIR}/${MOUNT}-migrate.log";
SAME_LOG="${LOG_DIR}/${MOUNT}-local.log";
ERR_LOG="${LOG_DIR}/${MOUNT}-error.log";

# Location of passwd file to determine local vs. NIS
PASSWD="/etc/passwd";
UIDMAP="/usr/local/migrate/uidmap";


check_local()
{
#	print "Checking to see if user is local"

	${AWKCMD} '
		BEGIN { FS = ":"; }
		{
#			print $0;
			if(cur_uid == $3)
			{
#				print "-------->", $0;
				local = 0;
				break;
			}
			else
			{
				next;
			}
		}
		END { exit local; }
	' cur_uid=${CUR_UID} local=1 ${PASSWD}
}

set_new_uid()
{
	${AWKCMD} '
	{
		if($1 == cur_uid)
		{
#			print "-------->", $0;
			newuid = $2;
			if(system("chown -h "newuid" "target) !=0)
			{
				exit 1;
			}
			break;
		}
		else
		{
			next;
		}
	}
	' cur_uid=${CUR_UID} target=${TARGET} ${UIDMAP}
}

chk_file()
{
	ls -ldn ${TARGET} >> ${ORIG_LOG};
	CUR_UID=`ls -ldn "${TARGET}" | awk '{print $3}'`;
}

# ###################################################################
# main() - Script begins here
# ###################################################################

# Check to see that awk is where we think it is
case `uname -s` in
	AIX )
	if [[ -f "/usr/bin/awk" ]]; then
		AWKCMD="/usr/bin/awk"
	else
		print "awk not installed as /usr/bin/awk";
		exit
	fi
	;;

	HP-UX )
	if [[ -f "/usr/bin/awk" ]]; then
		AWKCMD="/usr/bin/awk"
	else
		print "awk not installed as /usr/bin/awk";
		exit
	fi
	;;

	SunOS )
	if [[ -f "/usr/bin/nawk" ]]; then
		AWKCMD="/usr/bin/nawk"
	else
		print "awk not installed as /usr/bin/nawk";
		exit
	fi
	;;

	Linux )
	if [[ -f "/usr/bin/gawk" ]]; then
		AWKCMD="/usr/bin/gawk"
	else
		print "awk not installed as /usr/bin/gawk";
		exit
	fi
	if [[ -f "/usr/bin/sed" ]]; then
		SEDCMD="/usr/bin/sed"
	else
		print "sed not installed as /usr/bin/sed";
		exit
	fi
	;;

	* )
		echo "Unsupported OS";
		exit
	;;
esac

# Make the log directory
if [[ ! -d ${LOG_DIR} ]]; then
	/usr/bin/mkdir -p ${LOG_DIR}
fi

# Setup the log files
if [[ ! -f ${ORIG_LOG} ]]; then
	echo "Save log for uid migration - ${MOUNT}" > ${ORIG_LOG};
	echo "" >> ${ORIG_LOG};
fi
if [[ ! -f ${SAME_LOG} ]]; then
	echo "Files that have local UIDs in the passwd file that where unchanged - ${MOUNT}" > ${SAME_LOG};
	echo "" >> ${SAME_LOG};
fi
if [[ ! -f ${CHNG_LOG} ]]; then
	echo "Files that where changed to a new NIS UID - ${MOUNT}" > ${CHNG_LOG};
	echo "" >> ${CHNG_LOG};
fi
if [[ ! -f ${ERR_LOG} ]]; then
	echo "Error Log - ${MOUNT}" > ${ERR_LOG};
	echo "" >> ${ERR_LOG};
fi


chk_file;

if check_local
then
	# Local Account
	ls -ldn ${TARGET} >> ${SAME_LOG};
else
	# NIS Account
	if set_new_uid
	then
		ls -ldn ${TARGET} >> ${CHNG_LOG};
	else
		#Something bad happened :(
		echo "${TARGET} - Chown failed" >> ${ERR_LOG}
	fi
fi

